#include <tcl.h>
#include <tk.h>

#include <windows.h>

#include <errno.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>

#ifndef MAX_PATH
#define MAX_PATH	2048
#endif

char **argv;
int argc;

void Debug (char *string) {
	MessageBox(NULL, string, "Debug", MB_OK|MB_ICONINFORMATION);
}

int memerror() {
	fprintf(stderr, "Can't allocate enough memory. Bailing out\n");
	exit(1);
}

static void setargv(int *argcPtr, char ***argvPtr) {
    char *cmdLine, *p, *arg, *argSpace;
    char **argv;
    int argc, size, inquote, copy, slashes;
    
    cmdLine = GetCommandLine();

    size = 2;
    for (p = cmdLine; *p != '\0'; p++) {
	if ((*p == ' ') || (*p == '\t')) {
	    size++;
	    while ((*p == ' ') || (*p == '\t')) {
		p++;
	    }
	    if (*p == '\0') {
		break;
	    }
	}
    }
    argSpace = (char *) Tcl_Alloc(
	    (unsigned) (size * sizeof(char *) + strlen(cmdLine) + 1));
    argv = (char **) argSpace;
    argSpace += size * sizeof(char *);
    size--;

    p = cmdLine;
	for (argc = 0; argc < size; argc++) {
		argv[argc] = arg = argSpace;
		while ((*p == ' ') || (*p == '\t')) {
			p++;
		}
		if (*p == '\0') {
			break;
		}

		inquote = 0;
		slashes = 0;
		while (1) {
			copy = 1;
			while (*p == '\\') {
				slashes++;
				p++;
			}
			if (*p == '"') {
				if ((slashes & 1) == 0) {
					copy = 0;
					if ((inquote) && (p[1] == '"')) {
						p++;
						copy = 1;
					} else {
						inquote = !inquote;
					}
				}
				slashes >>= 1;
			}

			while (slashes) {
				*arg = '\\';
				arg++;
				slashes--;
			}

			if ((*p == '\0') || (!inquote && ((*p == ' ') || (*p == '\t')))) {
				break;
			}
			if (copy != 0) {
				*arg = *p;
				arg++;
			}
			p++;
		}
		*arg = '\0';
		argSpace = arg + 1;
	}
    argv[argc] = NULL;

    *argcPtr = argc;
    *argvPtr = argv;
}

int Init(Tcl_Interp* interp) {
    char *args;
    char buf[100];
    char script[1024];
    char tcl_dir [1024];
    char tk_dir [1024];
    char autopath [1024];
    char mainfile [1024];
    char prgpath [1024];
    char *p;

    char drv[MAX_PATH];
    char path[MAX_PATH];
    char fname[MAX_PATH];
    char ext[MAX_PATH];

    Tcl_FindExecutable(argv[0]);
    interp = Tcl_CreateInterp();

    if( Tcl_PkgRequire(interp, "Tcl", TCL_VERSION, 1)==0 ){
	    return 1;
    }

    args = Tcl_Merge(argc-1, (CONST84 char * CONST *)argv+1);
    Tcl_SetVar(interp, "argv", args, TCL_GLOBAL_ONLY);
    ckfree(args);
    sprintf(buf, "%d", argc-1);
    Tcl_SetVar(interp, "argc", buf, TCL_GLOBAL_ONLY);
    Tcl_SetVar(interp, "argv0", argv[0], TCL_GLOBAL_ONLY);
    Tcl_SetVar(interp, "tcl_interactive", "0", TCL_GLOBAL_ONLY);

     GetModuleFileName(NULL, mainfile, 1024);
     _splitpath(mainfile, drv, path, fname, ext);
	sprintf(prgpath, "%s%s", drv, path);
	
    sprintf(tcl_dir, "%s/tcl8.4", prgpath);
    sprintf(tk_dir, "%s/tk8.4", prgpath);

    Tcl_SetVar2(interp, "env", "TCL_LIBRARY", tcl_dir, TCL_GLOBAL_ONLY);
    Tcl_SetVar2(interp, "env", "TK_LIBRARY", tk_dir, TCL_GLOBAL_ONLY);

    if( Tcl_Init(interp) ) return TCL_ERROR;

    sprintf(autopath, " %s", tcl_dir);
    Tcl_SetVar(interp, "auto_path", autopath, TCL_GLOBAL_ONLY|TCL_APPEND_VALUE);
    
    Tcl_SetVar(interp, "tcl_libPath", tcl_dir, TCL_GLOBAL_ONLY);

    Tk_InitConsoleChannels(interp);
    if ( Tk_Init(interp) ) return TCL_ERROR;

    Tcl_StaticPackage(interp,"Tk", Tk_Init, 0);
    Tk_CreateConsoleWindow(interp);

    sprintf(script, "%stest.tcl", prgpath);
	//Debug(script);
	Tcl_EvalFile(interp, script);

    Tk_MainLoop();

    Tcl_DeleteInterp(interp);
    Tcl_Exit(0);

	return TCL_OK;
}

int WINAPI WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance,
  LPSTR lpCmdLine, int nCmdShow )
{
    setargv(&argc, &argv);
    
    Tk_Main(argc, argv, Init);

    return 0;
}
